@@ -2,14 +2,13 @@ from statefun_tasks.serialisation import DefaultSerialiser
 from statefun_tasks.pipeline_builder import PipelineBuilder
 
 from statefun_tasks.types import RetryPolicy, TaskAlreadyExistsException, \
-    TASK_STATE_TYPE, TASK_REQUEST_TYPE, TASK_RESULT_TYPE, TASK_EXCEPTION_TYPE, TASK_ACTION_REQUEST_TYPE, TASK_ACTION_RESULT_TYPE, TASK_ACTION_EXCEPTION_TYPE
+    TASK_STATE_TYPE, TASK_REQUEST_TYPE, TASK_RESULT_TYPE, TASK_EXCEPTION_TYPE, TASK_ACTION_REQUEST_TYPE, PIPELINE_STATE_TYPE
 
-from statefun_tasks.messages_pb2 import TaskRequest, TaskResult, TaskException, TaskActionRequest, TaskActionResult, TaskActionException, \
-    TaskAction, TaskStatus, TaskState
+from statefun_tasks.messages_pb2 import TaskRequest, TaskResult, TaskException, TaskAction
 
 from statefun_tasks.type_helpers import _create_task_result, _create_task_exception
 from statefun_tasks.context import TaskContext
-from statefun_tasks.utils import _gen_id, _task_type_for, _is_tuple, _type_name, _annotated_protos_for
+from statefun_tasks.utils import _task_type_for
 from statefun_tasks.pipeline import _Pipeline
 
 from statefun_tasks.tasks import _FlinkTask
@@ -18,15 +17,13 @@ from statefun_tasks.actions import _FlinkAction
 from statefun import ValueSpec, Context, Message
 from datetime import timedelta
 from typing import Union
-from functools import partial
 import logging
-import asyncio
 
 _log = logging.getLogger('FlinkTasks')
 
 
-def _run_pipeline(pipeline_proto, serialiser):
-    return PipelineBuilder.from_proto(pipeline_proto, serialiser)
+def _run_pipeline(pipeline_proto):
+    return PipelineBuilder.from_proto(pipeline_proto)
 
 
 class FlinkTasks(object):
@@ -54,9 +51,10 @@ class FlinkTasks(object):
             ValueSpec(name="task_result", type=TASK_RESULT_TYPE),
             ValueSpec(name="task_exception", type=TASK_EXCEPTION_TYPE),
             ValueSpec(name="task_state", type=TASK_STATE_TYPE),
+            ValueSpec(name="pipeline_state", type=PIPELINE_STATE_TYPE),
         ]
 
-        self.register_builtin('run_pipeline', partial(_run_pipeline, serialiser=self._serialiser))
+        self.register_builtin('run_pipeline', _run_pipeline)
 
     def value_specs(self):
         return self._value_specs
@@ -197,8 +195,7 @@ class FlinkTasks(object):
         return send_func(*args, **kwargs)
 
     def _get_pipeline(self, context):
-        state = context.get_state()
-        pipeline_protos = state.get('pipeline', None)
+        pipeline_protos = context.pipeline_state.pipeline
 
         if pipeline_protos is not None:
             return _Pipeline.from_proto(pipeline_protos, self._serialiser)
@@ -206,8 +203,12 @@ class FlinkTasks(object):
             raise ValueError(f'Missing pipleline for task_id - {context.get_task_id()}')
 
     def _resume_pipeline(self, context, task_result_or_exception: Union[TaskResult, TaskException]):
+        _log.info(f'Started {task_result_or_exception.type}, {context}')
+
         pipeline = self._get_pipeline(context)
         pipeline.resume(context, task_result_or_exception)
+        
+        _log.info(f'Finished {task_result_or_exception.type}, {context}')
 
     async def _invoke_task(self, context, task_request):
         _log.info(f'Started {task_request.type}, {context}')
@@ -256,8 +257,6 @@ class FlinkTasks(object):
         _log.info(f'Finished Action [{TaskAction.Name(action_request.action)}]')
 
     def _emit_result(self, context, task_input, task_result):
-        state = context.get_state()
-
         # either send a message to egress if reply_topic was specified
         if task_input.HasField('reply_topic'):
             context.send_egress_message(topic=task_input.reply_topic, value=task_result)
@@ -270,35 +269,32 @@ class FlinkTasks(object):
         elif isinstance(task_input, TaskRequest):
             # if this was a task request it could be part of a pipeline - i.e. called from another tasks so
             # either call back to original caller (e.g. if this is a result of a retry)
-            if isinstance(task_input, TaskRequest) and 'original_caller_address' in state and 'original_caller_id' in state:
-                context.send_message(state['original_caller_address'], state['original_caller_id'], task_result)
+            if context.task_state.original_caller_id != '':
+                context.send_message(context.task_state.original_caller_address, context.task_state.original_caller_id, task_result)
 
             # or call back to our caller (if there is one)
             elif context.get_caller_id() is not None:
                 context.send_message(context.get_caller_address(), context.get_caller_id(), task_result)
 
     def _attempt_retry(self, context, task_request, task_exception):
-        state = context.get_state()
-
-        if task_exception.maybe_retry and task_exception.retry_policy is not None:
-
-            retry_count = state.get('retry_count', 0)
-            if retry_count >= task_exception.retry_policy.max_retries:
+        if task_exception.maybe_retry and task_exception.retry_policy is not None:           
+            if context.task_state.retry_count >= task_exception.retry_policy.max_retries:
                 return False
 
             # save the original caller address and id for this task_request prior to calling back to ourselves which would overwrite
-            if not 'original_caller_id' in state or not 'original_caller_id' in state:
-                context.update_state({'original_caller_address': context.get_caller_address(), 'original_caller_id': context.get_caller_id()})
+            if context.task_state.original_caller_id == '':
+                context.task_state.original_caller_id = context.get_caller_id()
+                context.task_state.original_caller_address = context.get_caller_address()
 
             # remove previous task_request from state, increment retry count
             del context.storage.task_request
-            state['retry_count'] = retry_count + 1
+            context.task_state.retry_count += 1
 
             # send retry
             delay = timedelta(milliseconds=task_exception.retry_policy.delay_ms)
 
             if task_exception.retry_policy.exponential_back_off:
-                delay = timedelta(milliseconds=task_exception.retry_policy.delay_ms ^ retry_count)
+                delay = timedelta(milliseconds=task_exception.retry_policy.delay_ms ^ context.task_state.retry_count)
 
             if delay:
                 context.send_message_after(delay, context.get_address(), context.get_task_id(), task_request)
